@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## Module Generic - ~/lib/Module/Generic/File/Cache.pm
-## Version v0.2.0
+## Version v0.2.1
 ## Copyright(c) 2022 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2022/03/16
-## Modified 2022/07/18
+## Modified 2022/07/26
 ## All rights reserved
 ## 
 ## This program is free software; you can redistribute  it  and/or  modify  it
@@ -22,12 +22,13 @@ BEGIN
     use Nice::Try;
     use JSON 4.03 qw( -convert_blessed_universally );
     use Scalar::Util ();
-    use Storable 3.25 ();
+    # use Storable 3.25 ();
+    use Storable::Improved v0.1.3;
     # Hash of cache file to objects to maintain the shared cache file objects created
     $CACHE_REPO = [];
     $CACHE_TO_OBJECT = {};
     $DEBUG = 0;
-    our $VERSION = 'v0.2.0';
+    our $VERSION = 'v0.2.1';
 };
 
 use strict;
@@ -50,7 +51,8 @@ sub init
     $self->{size}       = 0;
     $self->{_init_strict_use_sub} = 1;
     # Storable keps breaking :(
-    # I leave the feature of using it as a choice to the user, but defaults to JSON
+    # I leave the feature of using it as a choice to the user, but defaults to JSON.
+    # Other possibilities could be cbor, sereal and storable
     $self->{_packing_method} = 'json';
     $self->SUPER::init( @_ ) || return( $self->pass_error );
     my $tmpdir = sys_tmpdir();
@@ -65,6 +67,8 @@ sub init
 
 sub binmode { return( shift->_set_get_scalar( 'binmode', @_ ) ); }
 
+sub cbor { return( shift->_packing_method( 'cbor' ) ); }
+
 sub create { return( shift->_set_get_boolean( 'create', @_ ) ); }
 
 sub delete { return( shift->remove( @_ ) ); }
@@ -98,6 +102,18 @@ sub flags
     return( $flags );
 }
 
+sub ftok
+{
+    my $self = shift( @_ );
+    my $id = shift( @_ );
+    my $f = sys_tmpdir();
+    return( int(
+                ( int( $f->inode ) & 0xffff ) |
+                int( ( $f->device ) << 16 ) |
+                ( ( $id & 0xff ) << 21 )
+               ) );
+}
+
 sub id { return( shift->_set_get_scalar( 'id', @_ ) ); }
 
 sub json { return( shift->_packing_method( 'json' ) ); }
@@ -285,6 +301,7 @@ sub read
     $self->messagef( 4, "${bytes} bytes read with a buffer now of size %d, unpacking data now.", CORE::length( $buffer ) );
     return( $self->pass_error( $file->error ) ) if( !defined( $bytes ) );
     my $packing = $self->_packing_method;
+    $packing = lc( $packing ) if( defined( $packing ) );
     my $data;
     if( CORE::length( $buffer ) )
     {
@@ -294,11 +311,21 @@ sub read
             {
                 $data = $self->_decode_json( $buffer );
             }
+            elsif( $packing eq 'cbor' )
+            {
+                $data = $self->deserialise( data => $buffer, serialiser => 'CBOR::XS', allow_sharing => 1 );
+            }
+            elsif( $packing eq 'sereal' )
+            {
+                $data = $self->deserialise( data => $buffer, serialiser => 'Sereal', freeze_callbacks => 1 );
+            }
+            # By default Storable::Improved
             else
             {
-                $data = Storable::thaw( $buffer );
+                # $data = Storable::Improved::thaw( $buffer );
+                $data = $self->deserialise( data => $buffer, serialiser => 'Storable::Improved' );
             }
-            $self->message( 4, "Decoded data '$buffer' -> '$data': ", sub{ $self->dump( $data ) });
+            $self->message( 4, "Decoded data '", ( $buffer // '' ), "' -> '", ( $data // '' ), "': ", sub{ $self->dump( $data ) });
         }
         catch( $e )
         {
@@ -399,8 +426,17 @@ sub reset
     return( $self );
 }
 
+sub sereal { return( shift->_packing_method( 'sereal' ) ); }
+
 sub serial { return( shift->_set_get_scalar( 'serial', @_ ) ); }
 
+sub serialiser { return( shift->_set_get_scalar( '_packing_method', @_ ) ); }
+
+{
+    no warnings 'once';
+    *serializer = \&serialiser;
+}
+
 sub size { return( shift->_set_get_scalar( 'size', @_ ) ); }
 
 sub stat
@@ -449,9 +485,28 @@ sub write
         {
             $encoded = $self->_encode_json( $data );
         }
+        elsif( $packing eq 'cbor' )
+        {
+            $encoded = $self->serialise( $data, serialiser => 'CBOR::XS', allow_sharing => 1 );
+            return( $self->pass_error ) if( !defined( $encoded ) );
+        }
+        elsif( $packing eq 'sereal' )
+        {
+            $self->_load_class( 'Sereal::Encoder' ) || return( $self->pass_error );
+            $encoded = $self->serialise( $data,
+                serialiser => 'Sereal',
+                freeze_callbacks => 1,
+                compress => &{"Sereal\::Encoder::SRL_ZLIB"}(),
+            );
+            return( $self->pass_error ) if( !defined( $encoded ) );
+        }
+        # Default to Storable::Improved
         else
         {
-            $encoded = Storable::freeze( $data );
+            # local $Storable::forgive_me = 1;
+            # $encoded = Storable::Improved::freeze( $data );
+            $encoded = $self->serialise( $data, serialiser => 'Storable::Improved' );
+            return( $self->pass_error ) if( !defined( $encoded ) );
         }
     }
     catch( $e )
@@ -621,18 +676,6 @@ sub _fill
     return(1);
 }
 
-sub ftok
-{
-    my $self = shift( @_ );
-    my $id = shift( @_ );
-    my $f = sys_tmpdir();
-    return( int(
-                ( int( $f->inode ) & 0xffff ) |
-                int( ( $f->device ) << 16 ) |
-                ( ( $id & 0xff ) << 21 )
-               ) );
-}
-
 sub _packing_method { return( shift->_set_get_scalar( '_packing_method', @_ ) ); }
 
 sub _str2key
@@ -778,7 +821,6 @@ END
     }
 };
 
-
 1;
 # NOTE: POD
 __END__
@@ -800,7 +842,7 @@ Module::Generic::File::Cache - File-based Cache
 
 =head1 VERSION
 
-    v0.2.0
+    v0.2.1
 
 =head1 DESCRIPTION
 
@@ -816,6 +858,10 @@ This instantiates a shared cache object. It takes the following parameters:
 
 =over 4
 
+=item I<cbor>
+
+Provided with a value (true or false does not matter), and this will set L<CBOR::XS> as the data serialisation mechanism when storing data to cache file.
+
 =item I<debug>
 
 A debug value will enable debugging output (equal or above 3 actually)
@@ -832,9 +878,9 @@ See L<perlmod> for more about object destruction.
 
 =item I<json>
 
-Provided with a value (true or false does not matter), and this will set L<JSON> as the data packing mechanism when storing data to memory.
+Provided with a value (true or false does not matter), and this will set L<JSON> as the data serialisation mechanism when storing data to cache file.
 
-Please note that if you want to store objects, you need to use I<storable> instead, because L<JSON> is not suitable to serialise objects.
+Please note that if you want to store objects, you need to use I<cbor>, I<sereal> or I<storable> instead, because L<JSON> is not suitable to serialise objects.
 
 =item I<key>
 
@@ -856,6 +902,10 @@ Shared cache files are owned by system users and access to shared cache files is
 
 If you do not want to share it with any other user than yourself, setting mode to C<0600> is fine.
 
+=item I<sereal>
+
+Provided with a value (true or false does not matter), and this will set L<Sereal> as the data serialisation mechanism when storing data to cache file.
+
 =item I<size>
 
 The size in byte of the shared cache.
@@ -864,7 +914,7 @@ This is set once it is created. You can create again the shared cache file with
 
 =item I<storable>
 
-Provided with a value (true or false does not matter), and this will set L<Storable> as the data packing mechanism when storing data to cache file.
+Provided with a value (true or false does not matter), and this will set L<Storable::Improved> as the data serialisation mechanism when storing data to cache file.
 
 =back
 
@@ -881,6 +931,10 @@ An object will be returned if it successfully initiated, or C<undef()> upon erro
 
 =head2 binmode
 
+=head2 cbor
+
+When called, this will set L<CBOR::XS> as the data serialisation mechanism when storing data to cache file or reading data from cache file.
+
 =head2 create
 
 Boolean value. If set, this will have L</open> create the cache file if it does not already exists.
@@ -1021,6 +1075,10 @@ This serial is created based on the I<key> parameter provided either upon object
 
 The serial is created by calling L</ftok> to provide a reliable and repeatable numeric identifier. L</ftok> is a simili polyfill of L<IPC::SysV/ftok>
 
+=head2 serialiser
+
+Sets or gets the serialiser. Possible values are: C<cbor>, C<json>, C<sereal>, C<storable>
+
 =head2 size
 
 Sets or gets the shared cache size.
