@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## Module Generic - ~/lib/Module/Generic/SharedMem.pm
-## Version v0.3.0
+## Version v0.3.1
 ## Copyright(c) 2022 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2021/01/18
-## Modified 2022/07/18
+## Modified 2022/07/26
 ## All rights reserved
 ## 
 ## This program is free software; you can redistribute  it  and/or  modify  it
@@ -109,7 +109,7 @@ EOT
             lock    => [qw( LOCK_EX LOCK_SH LOCK_NB LOCK_UN )],
             'flock' => [qw( LOCK_EX LOCK_SH LOCK_NB LOCK_UN )],
     );
-    our $VERSION = 'v0.3.0';
+    our $VERSION = 'v0.3.1';
 };
 
 # use strict;
@@ -234,8 +234,10 @@ sub exists
         # $self->message( 3, "Found the shared memory? ", defined( $semid ) ? 'yes' : 'no' );
         if( defined( $semid ) )
         {
-            my $found = semctl( $semid, SEM_MARKER, IPC::SysV::GETVAL, 0 );
-            semctl( $semid, 0, IPC::SysV::IPC_RMID, 0 );
+            my $arg = 0;
+            my $found = semctl( $semid, SEM_MARKER, IPC::SysV::GETVAL, $arg );
+            $arg = 0;
+            semctl( $semid, 0, IPC::SysV::IPC_RMID, $arg );
             return( $found == SHM_EXISTS ? 1 : 0 );
         }
         else
@@ -248,7 +250,8 @@ sub exists
     catch( $e )
     {
         # $self->message( 3, "Trying to access shared memory triggered error: $e" );
-        semctl( $semid, 0, IPC::SysV::IPC_RMID, 0 ) if( $semid );
+        my $arg = 0;
+        semctl( $semid, 0, IPC::SysV::IPC_RMID, $arg ) if( $semid );
         return(0);
     }
 }
@@ -503,7 +506,8 @@ sub pid
     my $semid = $self->semid ||
         return( $self->error( "No semaphore set yet. You must open the shared memory first to remove semaphore." ) );
     no strict 'subs';
-    my $v = semctl( $semid, $sem, IPC::SysV::GETPID, 0 );
+    my $arg = 0;
+    my $v = semctl( $semid, $sem, IPC::SysV::GETPID, $arg );
     return( $v ? 0 + $v : undef() );
 }
 
@@ -557,17 +561,17 @@ sub read
             }
             elsif( $packing eq 'cbor' )
             {
-                $data = $self->deserialise( $buffer, serialiser => 'CBOR::XS', allow_sharing => 1 );
+                $data = $self->deserialise( data => $buffer, serialiser => 'CBOR::XS', allow_sharing => 1 );
             }
             elsif( $packing eq 'sereal' )
             {
-                $data = $self->deserialise( $buffer, serialiser => 'Sereal' );
+                $data = $self->deserialise( data => $buffer, serialiser => 'Sereal', freeze_callbacks => 1 );
             }
             # By default Storable::Improved
             else
             {
                 # $data = Storable::Improved::thaw( $buffer );
-                $data = $self->deserialise( $buffer, serialiser => 'Storable::Improved' );
+                $data = $self->deserialise( data => $buffer, serialiser => 'Storable::Improved' );
             }
             $self->message( 4, "Decoded data '$buffer' -> '$data': ", sub{ $self->dump( $data ) });
         }
@@ -612,10 +616,11 @@ sub remove
     }
     # Remove semaphore
     my $rv;
-    if( !defined( $rv = semctl( $semid, 0, IPC::SysV::IPC_RMID, 0 ) ) )
+    my $arg = 0;
+    if( !defined( $rv = semctl( $semid, 0, IPC::SysV::IPC_RMID, $arg ) ) )
     {
         $self->message( 3, "Failed to remove semaphore with id '$semid': $!" );
-        $self->error( "Warning only: could not remove the semaphore id \"$semid\": $!" );
+        warn( "Warning only: could not remove the semaphore id \"$semid\": $!" ) if( $self->_warnings_is_enabled );
     }
     $self->removed( $rv ? 1 : 0 );
     if( $rv )
@@ -647,10 +652,11 @@ sub remove_semaphore
     $self->unlock();
     my $rv;
     no strict 'subs';
-    if( !defined( $rv = semctl( $semid, 0, IPC::SysV::IPC_RMID, 0 ) ) )
+    my $arg = 0;
+    if( !defined( $rv = semctl( $semid, 0, IPC::SysV::IPC_RMID, $arg ) ) )
     {
         $self->message( 3, "Failed to remove semaphore with id '$semid': $!" );
-        $self->error( "Warning only: could not remove the semaphore id \"$semid\" with IPC::SysV::IPC_RMID value '", IPC::SysV::IPC_RMID, "': $!" );
+        warn( "Warning only: could not remove the semaphore id \"$semid\" with IPC::SysV::IPC_RMID value '", IPC::SysV::IPC_RMID, "': $!" ) if( $self->_warnings_is_enabled );
     }
     $self->removed_semaphore( $rv ? 1 : 0 );
     $self->semid( undef() );
@@ -685,6 +691,13 @@ sub sereal { return( shift->_packing_method( 'sereal' ) ); }
 
 sub serial { return( shift->_set_get_scalar( 'serial', @_ ) ); }
 
+sub serialiser { return( shift->_set_get_scalar( '_packing_method', @_ ) ); }
+
+{
+    no warnings 'once';
+    *serializer = \&serialiser;
+}
+
 sub shmstat
 {
     my $self = shift( @_ );
@@ -710,7 +723,8 @@ sub stat
         {
             my $sem = shift( @_ );
             # $self->message( 3, "Retrieving semaphore value for '$sem'" );
-            my $v = semctl( $id, $sem, IPC::SysV::GETVAL, 0 );
+            my $arg = 0;
+            my $v = semctl( $id, $sem, IPC::SysV::GETVAL, $arg );
             return( $v ? 0 + $v : undef() );
         }
         else
@@ -792,7 +806,12 @@ sub write
         }
         elsif( $packing eq 'sereal' )
         {
-            $encoded = $self->serialise( $data, serialiser => 'Sereal' );
+            $self->_load_class( 'Sereal::Encoder' ) || return( $self->pass_error );
+            $encoded = $self->serialise( $data,
+                serialiser => 'Sereal',
+                freeze_callbacks => 1,
+                compress => &{"Sereal\::Encoder::SRL_ZLIB"}(),
+            );
             return( $self->pass_error ) if( !defined( $encoded ) );
         }
         # Default to Storable::Improved
